/*******************************************************************/
/*

 *TEMP BUG FIXED 03/28/03 MS/ME									
Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

Demofile for 100 MBit Ethernet Server. 

The program transfers images from the camera to the PC. It transfers
up to 7 images (640*480 Byte) per second. Please start the program 
"image" on the camera at Telnet port 23. Then start the PC program.

After you start the camera program the camera waits at port 
SERVER_PORT (port 2002) for a string RECIEVE_BYTES (24 Bytes).

This string includes the following data:

1. x position of the image (int value, 4 Bytes)
2. y position of the image
3. dx
4. dy
5. incrx 
6. incry

If the string is complete the camera starts transfering the image.
It response the string values at port 23. Then it waits again at
port SERVER_PORT for a new string. Is the value dx=0 the program ends.

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <licence.h>

/*******************************************************************/

#define RECEIVE_BYTES     24  /* int value (4 Bytes) for x, y, dx, dy, incrx, incry */
#define SERVER_PORT     2002  /* waiting at port 2002 for any connection */
#define MEMORY        3072000  /* memory to store one image */

/*******************************************************************/

void main(int argc, char *argv)
{
   unsigned char *linebuffer, *plinebuf, *pixel;
   int  receivedata[RECEIVE_BYTES/4]; 
   int  i, j, x, y, dx, dy, incrx, incry, result, ms, count, temp, nr=0;
   long sec;
   sockaddr_in    laddr, raddr;
   unsigned       sock, listensock;
   unsigned       error;
   uint_16        rlen;



   INIT_LICENCE();



   /* transfer the image from the physical page */
   ScrSetLogPage((int)ScrGetPhysPage);


   /* temporarily work around */
    vmode(1);
 
 //  tpict();
//   setvar(DISP_ACTIVE,0);

   /* allocate memory for image raw data */
   linebuffer=(unsigned char *)sysmalloc(MEMORY,3);
   if((int)linebuffer==0) print("not enough memory\n");

   setvar(CAPT_START,(int)linebuffer);

   setvar(VWIDTH,1024);

   /* server services port */
   laddr.sin_family      = AF_INET;
   laddr.sin_port        = SERVER_PORT;   /* server listen at port SERVER_PORT (2002) */
   laddr.sin_addr.s_addr = INADDR_ANY;    /* every IP addr can connect to the camera */


   /* use TCP/IP protocoll and search for a free port */
   sock = socket_stream();

   if (sock == VCRT_SOCKET_ERROR) 
     {
     print("\nCreate stream socket failed");
     return;
     } 

   /* sends immediatly the data */    
//   status = setsockopt(sock, SOL_TCP, OPT_SEND_NOWAIT, &opt_value, opt_length);
//   if (status != VCRT_OK)
//     {
//	stats->last_error = SET_SOCK_OPTION;
//	if (config->debug)
//	print("Could not set socket option: OPT_SEND_NOWAIT!\n");
//     }


   /* connection between free port and selected SERVER_PORT. Now SERVER_PORT is free again for new connections. */
   error = bind(sock, &laddr, sizeof(laddr));

   if (error != VCRT_OK) 
     {
     print("\nStream bind failed - 0x%x", error);
     return;
     }
 
   /* activate sock for TCP/IP connections */
   error = listen(sock, 0);

   if (error != VCRT_OK) 
     {
     print("\nListen failed - 0x%x", error);
     return;
     }

   listensock = sock;

   print("\n\nFast Ethernet port %d\n",laddr.sin_port);

   for (;;) {

      /* Are there connections at any port? The number 0 means: Wait until connection */
      sock = VCRT_selectall(0);

      /* Is there one who wants to connect to port SERVER_PORT */
      if (sock == listensock)
      {
        /* Connection requested; accept it */
        rlen = sizeof(raddr);

        /* accept connection */
        print("\nwait for accept");
        sock = accept(listensock, &raddr, &rlen);

        if (sock == VCRT_SOCKET_ERROR)
        {
          print("\nAccept failed, error 0x%x",VCRT_geterror(listensock));
          continue;
        }
        else
        {
          print("\naccept ready");
        }

        print("\nConnection from %d.%d.%d.%d, port %d",
           (raddr.sin_addr.s_addr >> 24) & 0xFF,
           (raddr.sin_addr.s_addr >> 16) & 0xFF,
           (raddr.sin_addr.s_addr >>  8) & 0xFF,
            raddr.sin_addr.s_addr        & 0xFF,
            raddr.sin_port);


        do
        {
//          print("\n\nWaiting for image size");

          count = recv(sock, (char *)receivedata, RECEIVE_BYTES, 0); 
          dx=0;

          if(count == VCRT_ERROR)
            print("\nVCRT_ERROR 0x%x",VCRT_geterror(sock));
          else
            if(count < RECEIVE_BYTES)
               print("\nonly %d bytes read",count);
          else /* no errors*/
          { 
          x=receivedata[0];
          y=receivedata[1];
          dx=receivedata[2];
          dy=receivedata[3];
          incrx=receivedata[4];
          incry=receivedata[5];
          }

 //         print("\nx=%d y=%d dx=%d dy=%d incrx=%d incry=%d nr=%d", x,y,dx,dy,incrx,incry, nr++);

        /* dx=0 : quit program */
        if (dx)
        {
//          tpict();

          plinebuf=linebuffer+28; /* 4 Bytes for length and 24 Bytes for image data */

          count=0;

          for(j=0;j<dy;j+=incry)
          {
            pixel=(unsigned char *)ScrByteAddr(x,y+j);

            for(i=0;i<dx;i+=incrx) 
            {
              *plinebuf++ = *pixel;
               pixel+=incrx;
               count++;
            }
          }

          plinebuf=linebuffer;

          count+=RECEIVE_BYTES; 
//          temp=count;
//          pixel=(unsigned char *)temp;
//          for(i=0;i<4;i++) 
//            {
//              *plinebuf++ = *pixel++;
//            }
		  *((int *)plinebuf) = count;
		  plinebuf += 4;

          pixel=(unsigned char *)receivedata;
          for(i=0;i<RECEIVE_BYTES;i++) 
            {
              *plinebuf++ = *pixel++;
            }

          sec = getvar(SEC);
          ms  = getvar(MSEC);
           
          result=send(sock, (char *)linebuffer, count, 0);
          if(result==VCRT_ERROR) 
          {
            print("\nSend VCRT_ERROR=%lx",VCRT_geterror(sock));
          }
          else
          {
            if(result!=count)    
            print("\nSend error wrong length=%d result=%d",count,result);
          }

          sec = getvar(SEC)-sec;
          ms  = getvar(MSEC)-ms;
            
          ms = 1000*(int)sec + ms;
//          print("  t=%dms",ms);
        }
        }
        while(dx);

        shutdown(sock, FLAG_CLOSE_TX);
        };

     break;

   } /* Endfor */
   shutdown(listensock, FLAG_CLOSE_TX);

	   sysfree(linebuffer);


   /* activate image display */
//  setvar(DISP_ACTIVE,1);

vmode(vmLive);

} /* Endbody */


/* EOF */
